with Ada.Numerics,
     Ada.Numerics.Elementary_Functions,
     Ada.Numerics.Float_Random,
     Display;
use Ada.Numerics, Ada.Numerics.Elementary_Functions;

procedure Tube is
  N       : constant := 400; -- number of points to display at once
  Mono,
  Stereo  : Display.Cloud_Display_Type(N);
  subtype Indices is Integer range 1 .. N;
  Theta   : array (Indices) of Float;
  Z       : array (Indices) of Float;
  Atom_List: Display.Point_Lists(Indices);
  Rotation: Float;
  type Days is mod 365;
  D       : Days := 0;
  Rgen    : Ada.Numerics.Float_Random.Generator;
begin
  for I in Indices loop
    Theta(I) := Ada.Numerics.Float_Random.Random(Rgen) * 2.0 * Pi;
    Z(I) := Ada.Numerics.Float_Random.Random(Rgen);
  end loop;
  Display.Open(Mono, "Deep Ocean Life Form",
    Position=>(Left=>0, Right=>300, Top=>0, Bottom=>300));
  Display.Open(Stereo, "Blue-Red Stereo", Stereo => True,
    Position=>(Left=>310, Right=>610, Top=>0, Bottom=>300));
  while not (Display.Is_Closed(Mono) and Display.Is_Closed(Stereo)) loop
    Rotation := Float(D) * 2.0 * Pi / 365.0;
    for I in Indices loop
      Atom_List(I) := (X => 0.5 + Sin(Theta(I) + Rotation) / 3.0,
                       Y => 0.5 + Cos(Theta(I) + Rotation) / 3.0,
                       Z => Z(I));
    end loop;
    Display.Brighten(Stereo, Atom_List);
    Display.Brighten(Mono, Atom_List);
    delay 0.01;
    D := D + 1;
  end loop;
end Tube;
